@@ -12,7 +12,7 @@
 # under the License.
 
 """
-Build open standard audit information based on incoming requests
+Build open standard audit information based on incoming requests.
 
 AuditMiddleware filter should be placed after keystonemiddleware.auth_token
 in the pipeline so that it can utilise the information the Identity server
@@ -28,7 +28,7 @@ import re
 import sys
 
 from oslo_config import cfg
-from oslo_context import context
+from oslo_context import context as oslo_context
 try:
     import oslo_messaging
     messaging = True
@@ -56,6 +56,27 @@ from keystonemiddleware.i18n import _LE, _LI
 
 _LOG = None
 
+_AUDIT_OPTS = [
+    cfg.StrOpt('driver',
+               default=None,
+               help='The Driver to handle sending notifications. Possible '
+                    'values are messaging, messagingv2, routing, log, test, '
+                    'noop. If not specified, then value from '
+                    'oslo_messaging_notifications conf section is used.'),
+    cfg.ListOpt('topics',
+                default=None,
+                help='List of AMQP topics used for OpenStack notifications. If'
+                     ' not specified, then value from '
+                     ' oslo_messaging_notifications conf section is used.'),
+    cfg.StrOpt('transport_url',
+               default=None,
+               secret=True,
+               help='A URL representing messaging driver to use for '
+                    'notification. If not specified, we fall back to the same '
+                    'configuration used for RPC.'),
+]
+cfg.CONF.register_opts(_AUDIT_OPTS, group="audit_middleware_notifications")
+
 
 def _log_and_ignore_error(fn):
     @functools.wraps(fn)
@@ -209,8 +230,7 @@ class OpenStackAuditApi(object):
                 endp['type'],
                 taxonomy.UNKNOWN),
             name=endp['name'],
-            id=identifier.norm_ns(endp['endpoints'][0].get('id',
-                                                           endp['name'])),
+            id=endp['endpoints'][0].get('id', endp['name']),
             admin_endp=endpoint.Endpoint(
                 name='admin',
                 url=endp['endpoints'][0].get('adminURL', taxonomy.UNKNOWN)),
@@ -224,7 +244,7 @@ class OpenStackAuditApi(object):
         return service
 
     def _build_typeURI(self, req, service_type):
-        """Build typeURI of target
+        """Build typeURI of target.
 
         Combines service type and corresponding path for greater detail.
         """
@@ -255,7 +275,7 @@ class OpenStackAuditApi(object):
         return target
 
     def get_target_resource(self, req):
-        """Retrieve target information
+        """Retrieve target information.
 
         If discovery is enabled, target will attempt to retrieve information
         from service catalog. If not, the information will be taken from
@@ -310,6 +330,8 @@ class KeystoneCredential(credential.Credential):
 class PycadfAuditApiConfigError(Exception):
     """Error raised when pyCADF fails to configure correctly."""
 
+    pass
+
 
 class AuditMiddleware(object):
     """Create an audit event based on request/response.
@@ -346,17 +368,21 @@ class AuditMiddleware(object):
 
         transport_aliases = self._get_aliases(cfg.CONF.project)
         if messaging:
+            transport = oslo_messaging.get_transport(
+                cfg.CONF,
+                url=cfg.CONF.audit_middleware_notifications.transport_url,
+                aliases=transport_aliases)
             self._notifier = oslo_messaging.Notifier(
-                oslo_messaging.get_transport(cfg.CONF,
-                                             aliases=transport_aliases),
-                os.path.basename(sys.argv[0]))
+                transport,
+                os.path.basename(sys.argv[0]),
+                driver=cfg.CONF.audit_middleware_notifications.driver,
+                topics=cfg.CONF.audit_middleware_notifications.topics)
 
     def _emit_audit(self, context, event_type, payload):
-        """Emit audit notification
+        """Emit audit notification.
 
         if oslo.messaging enabled, send notification. if not, log event.
         """
-
         if messaging:
             self._notifier.info(context, event_type, payload)
         else:
@@ -371,13 +397,13 @@ class AuditMiddleware(object):
 
         initiator = ClientResource(
             typeURI=taxonomy.ACCOUNT_USER,
-            id=identifier.norm_ns(str(req.environ['HTTP_X_USER_ID'])),
+            id=req.environ['HTTP_X_USER_ID'],
             name=req.environ['HTTP_X_USER_NAME'],
             host=host.Host(address=req.client_addr, agent=req.user_agent),
             credential=KeystoneCredential(
                 token=req.environ['HTTP_X_AUTH_TOKEN'],
                 identity_status=req.environ['HTTP_X_IDENTITY_STATUS']),
-            project_id=identifier.norm_ns(req.environ['HTTP_X_PROJECT_ID']))
+            project_id=req.environ['HTTP_X_PROJECT_ID'])
         target = self._cadf_audit.get_target_resource(req)
 
         event = factory.EventFactory().new_event(
@@ -398,8 +424,8 @@ class AuditMiddleware(object):
     def _process_request(self, request):
         event = self._create_event(request)
 
-        self._emit_audit(context.get_admin_context().to_dict(),
-                         'audit.http.request', event.as_dict())
+        self._emit_audit(request.context, 'audit.http.request',
+                         event.as_dict())
 
     @_log_and_ignore_error
     def _process_response(self, request, response=None):
@@ -425,14 +451,19 @@ class AuditMiddleware(object):
                 reporter=resource.Resource(id='target'),
                 reporterTime=timestamp.get_utc_now()))
 
-        self._emit_audit(context.get_admin_context().to_dict(),
-                         'audit.http.response', event.as_dict())
+        self._emit_audit(request.context, 'audit.http.response',
+                         event.as_dict())
 
     @webob.dec.wsgify
     def __call__(self, req):
         if req.method in self._ignore_req_list:
             return req.get_response(self._application)
 
+        # Cannot use a RequestClass on wsgify above because the `req` object is
+        # a `WebOb.Request` when this method is called so the RequestClass is
+        # ignored by the wsgify wrapper.
+        req.context = oslo_context.get_admin_context().to_dict()
+
         self._process_request(req)
         try:
             response = req.get_response(self._application)
@@ -445,7 +476,7 @@ class AuditMiddleware(object):
 
 
 def filter_factory(global_conf, **local_conf):
-    """Returns a WSGI filter app for use with paste.deploy."""
+    """Return a WSGI filter app for use with paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
 
