@@ -20,7 +20,7 @@
 # See them for their copyright.
 
 """
-S3 Token Middleware
+S3 Token Middleware.
 
 This WSGI component:
 
@@ -40,7 +40,7 @@ import requests
 import six
 from six.moves import urllib
 
-from keystonemiddleware.i18n import _, _LI
+from keystonemiddleware.i18n import _, _LI, _LW
 
 
 PROTOCOL_NAME = 'S3 Token Authentication'
@@ -109,12 +109,19 @@ class S3Token(object):
         self._reseller_prefix = conf.get('reseller_prefix', 'AUTH_')
         # where to find the auth service (we use this to validate tokens)
 
-        auth_host = conf.get('auth_host')
-        auth_port = int(conf.get('auth_port', 35357))
-        auth_protocol = conf.get('auth_protocol', 'https')
+        self._request_uri = conf.get('auth_uri')
+        if not self._request_uri:
+            self._logger.warning(_LW(
+                "Use of the auth_host, auth_port, and auth_protocol "
+                "configuration options was deprecated in the Newton release "
+                "in favor of auth_uri. These options may be removed in a "
+                "future release."))
+            auth_host = conf.get('auth_host')
+            auth_port = int(conf.get('auth_port', 35357))
+            auth_protocol = conf.get('auth_protocol', 'https')
 
-        self._request_uri = '%s://%s:%s' % (auth_protocol, auth_host,
-                                            auth_port)
+            self._request_uri = '%s://%s:%s' % (auth_protocol, auth_host,
+                                                auth_port)
 
         # SSL
         insecure = strutils.bool_from_string(conf.get('insecure', False))
@@ -251,6 +258,8 @@ class S3Token(object):
 
         req.headers['X-Auth-Token'] = token_id
         tenant_to_connect = force_tenant or tenant['id']
+        if six.PY2 and isinstance(tenant_to_connect, six.text_type):
+            tenant_to_connect = tenant_to_connect.encode('utf-8')
         self._logger.debug('Connecting with tenant: %s', tenant_to_connect)
         new_tenant_name = '%s%s' % (self._reseller_prefix, tenant_to_connect)
         environ['PATH_INFO'] = environ['PATH_INFO'].replace(account,
@@ -259,7 +268,7 @@ class S3Token(object):
 
 
 def filter_factory(global_conf, **local_conf):
-    """Returns a WSGI filter app for use with paste.deploy."""
+    """Return a WSGI filter app for use with paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
 
